@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2016 The Qt Company Ltd.
+** Copyright (C) 2017 The Qt Company Ltd.
 ** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the Qt Quick Templates 2 module of the Qt Toolkit.
@@ -46,6 +46,7 @@
 #include <QtQuickTemplates2/private/qquickcombobox_p.h>
 #include <QtQuickTemplates2/private/qquickcontrol_p.h>
 #include <QtQuickTemplates2/private/qquickcontainer_p.h>
+#include <QtQuickTemplates2/private/qquickdelaybutton_p.h>
 #include <QtQuickTemplates2/private/qquickdial_p.h>
 #include <QtQuickTemplates2/private/qquickdialog_p.h>
 #include <QtQuickTemplates2/private/qquickdialogbuttonbox_p.h>
@@ -69,6 +70,7 @@
 #include <QtQuickTemplates2/private/qquickroundbutton_p.h>
 #include <QtQuickTemplates2/private/qquickscrollbar_p.h>
 #include <QtQuickTemplates2/private/qquickscrollindicator_p.h>
+#include <QtQuickTemplates2/private/qquickscrollview_p.h>
 #include <QtQuickTemplates2/private/qquickshortcutcontext_p_p.h>
 #include <QtQuickTemplates2/private/qquickslider_p.h>
 #include <QtQuickTemplates2/private/qquickspinbox_p.h>
@@ -86,7 +88,9 @@
 #include <QtQuickTemplates2/private/qquicktoolbutton_p.h>
 #include <QtQuickTemplates2/private/qquicktoolseparator_p.h>
 #include <QtQuickTemplates2/private/qquicktooltip_p.h>
+#if QT_CONFIG(quick_listview) && QT_CONFIG(quick_pathview)
 #include <QtQuickTemplates2/private/qquicktumbler_p.h>
+#endif
 
 static inline void initResources()
 {
@@ -95,10 +99,12 @@ static inline void initResources()
 #endif
 }
 
+#if QT_CONFIG(shortcut)
 // qtdeclarative/src/quick/util/qquickshortcut.cpp
 typedef bool (*ShortcutContextMatcher)(QObject *, Qt::ShortcutContext);
 extern ShortcutContextMatcher qt_quick_shortcut_context_matcher();
 extern void qt_quick_set_shortcut_context_matcher(ShortcutContextMatcher matcher);
+#endif
 
 QT_BEGIN_NAMESPACE
 
@@ -114,25 +120,33 @@ public:
     void registerTypes(const char *uri);
 
 private:
+#if QT_CONFIG(shortcut)
     ShortcutContextMatcher originalContextMatcher;
+#endif
 };
 
 QtQuickTemplates2Plugin::QtQuickTemplates2Plugin(QObject *parent) : QQmlExtensionPlugin(parent)
 {
     initResources();
 
+#if QT_CONFIG(shortcut)
     originalContextMatcher = qt_quick_shortcut_context_matcher();
     qt_quick_set_shortcut_context_matcher(QQuickShortcutContext::matcher);
+#endif
 }
 
 QtQuickTemplates2Plugin::~QtQuickTemplates2Plugin()
 {
+#if QT_CONFIG(shortcut)
     qt_quick_set_shortcut_context_matcher(originalContextMatcher);
+#endif
 }
 
 void QtQuickTemplates2Plugin::registerTypes(const char *uri)
 {
-    // QtQuick.Templates 2.0 (Qt 5.7)
+    qmlRegisterModule(uri, 2, QT_VERSION_MINOR - 7); // Qt 5.7->2.0, 5.8->2.1, 5.9->2.2...
+
+    // QtQuick.Templates 2.0 (originally introduced in Qt 5.7)
     qmlRegisterType<QQuickAbstractButton>(uri, 2, 0, "AbstractButton");
     qmlRegisterType<QQuickApplicationWindow>(uri, 2, 0, "ApplicationWindow");
     qmlRegisterType<QQuickApplicationWindowAttached>();
@@ -171,7 +185,7 @@ void QtQuickTemplates2Plugin::registerTypes(const char *uri)
     qmlRegisterType<QQuickSpinBox>(uri, 2, 0, "SpinBox");
     qmlRegisterType<QQuickSpinButton>();
     qmlRegisterType<QQuickStackView>(uri, 2, 0, "StackView");
-    qmlRegisterType<QQuickStackAttached>();
+    qmlRegisterType<QQuickStackViewAttached>();
     qmlRegisterType<QQuickSwipeDelegate>(uri, 2, 0, "SwipeDelegate");
     qmlRegisterType<QQuickSwipe>();
     qmlRegisterType<QQuickSwipeViewAttached>();
@@ -187,8 +201,10 @@ void QtQuickTemplates2Plugin::registerTypes(const char *uri)
     qmlRegisterType<QQuickToolButton>(uri, 2, 0, "ToolButton");
     qmlRegisterType<QQuickToolTipAttached>();
     qmlRegisterType<QQuickToolTip>(uri, 2, 0, "ToolTip");
+#if QT_CONFIG(quick_listview) && QT_CONFIG(quick_pathview)
     qmlRegisterType<QQuickTumblerAttached>();
     qmlRegisterType<QQuickTumbler>(uri, 2, 0, "Tumbler");
+#endif
 
     // NOTE: register the latest revisions of all template/control base classes to
     // make revisioned properties available to their subclasses (synced with Qt 5.7)
@@ -197,8 +213,9 @@ void QtQuickTemplates2Plugin::registerTypes(const char *uri)
     qmlRegisterRevision<QQuickText, 6>(uri, 2, 0);
     qmlRegisterRevision<QQuickTextInput, 7>(uri, 2, 0);
     qmlRegisterRevision<QQuickTextEdit, 7>(uri, 2, 0);
+    qmlRegisterRevision<QWindow, 3>(uri, 2, 0);
 
-    // QtQuick.Templates 2.1 (Qt 5.8)
+    // QtQuick.Templates 2.1 (new types and revisions in Qt 5.8)
     qmlRegisterType<QQuickButtonGroup, 1>(uri, 2, 1, "ButtonGroup");
     qmlRegisterType<QQuickComboBox, 1>(uri, 2, 1, "ComboBox");
     qmlRegisterType<QQuickContainer, 1>(uri, 2, 1, "Container");
@@ -209,7 +226,7 @@ void QtQuickTemplates2Plugin::registerTypes(const char *uri)
     qmlRegisterType<QQuickPage, 1>(uri, 2, 1, "Page");
     qmlRegisterType<QQuickPopup, 1>(uri, 2, 1, "Popup");
     qmlRegisterType<QQuickRangeSlider, 1>(uri, 2, 1, "RangeSlider");
-    qmlRegisterType<QQuickRoundButton, 1>(uri, 2, 1, "RoundButton");
+    qmlRegisterType<QQuickRoundButton>(uri, 2, 1, "RoundButton");
     qmlRegisterType<QQuickSlider, 1>(uri, 2, 1, "Slider");
     qmlRegisterType<QQuickSpinBox, 1>(uri, 2, 1, "SpinBox");
     qmlRegisterType<QQuickStackView, 1>(uri, 2, 1, "StackView");
@@ -218,7 +235,33 @@ void QtQuickTemplates2Plugin::registerTypes(const char *uri)
     qmlRegisterType<QQuickTextArea, 1>(uri, 2, 1, "TextArea");
     qmlRegisterType<QQuickTextField, 1>(uri, 2, 1, "TextField");
     qmlRegisterType<QQuickToolSeparator>(uri, 2, 1, "ToolSeparator");
+#if QT_CONFIG(quick_listview) && QT_CONFIG(quick_pathview)
     qmlRegisterType<QQuickTumbler, 1>(uri, 2, 1, "Tumbler");
+#endif
+
+    // QtQuick.Templates 2.2 (new types and revisions in Qt 5.9)
+    qmlRegisterType<QQuickAbstractButton, 2>(uri, 2, 2, "AbstractButton");
+    qmlRegisterType<QQuickComboBox, 2>(uri, 2, 2, "ComboBox");
+    qmlRegisterType<QQuickDelayButton>(uri, 2, 2, "DelayButton");
+    qmlRegisterType<QQuickDial, 2>(uri, 2, 2, "Dial");
+    qmlRegisterType<QQuickDrawer, 2>(uri, 2, 2, "Drawer");
+    qmlRegisterType<QQuickRangeSlider, 2>(uri, 2, 2, "RangeSlider");
+    qmlRegisterType<QQuickScrollBar, 2>(uri, 2, 2, "ScrollBar");
+    qmlRegisterType<QQuickScrollView>(uri, 2, 2, "ScrollView");
+    qmlRegisterType<QQuickSlider, 2>(uri, 2, 2, "Slider");
+    qmlRegisterType<QQuickSpinBox, 2>(uri, 2, 2, "SpinBox");
+    qmlRegisterType<QQuickSwipeDelegate, 2>(uri, 2, 2, "SwipeDelegate");
+    qmlRegisterType<QQuickSwipeView, 2>(uri, 2, 2, "SwipeView");
+    qmlRegisterType<QQuickTabBar, 2>(uri, 2, 2, "TabBar");
+#if QT_CONFIG(quick_listview) && QT_CONFIG(quick_pathview)
+    qmlRegisterType<QQuickTumbler, 2>(uri, 2, 2, "Tumbler");
+#endif
+
+    // NOTE: register the latest revisions of all template/control base classes to
+    // make revisioned properties available to their subclasses (synced with Qt 5.9)
+    qmlRegisterRevision<QQuickText, 9>(uri, 2, 2);
+    qmlRegisterRevision<QQuickTextInput, 9>(uri, 2, 2);
+    qmlRegisterRevision<QQuickWindowQmlImpl, 2>(uri, 2, 2);
 }
 
 QT_END_NAMESPACE
