@@ -48,10 +48,10 @@
 #include "qquickpopup_p.h"
 #include "qquickpopupitem_p_p.h"
 #include "qquickapplicationwindow_p.h"
+#include "qquickdeferredexecute_p_p.h"
 
 #include <QtGui/private/qguiapplication_p.h>
 #include <QtGui/qpa/qplatformtheme.h>
-#include <QtQml/private/qqmlincubator_p.h>
 
 #if QT_CONFIG(accessibility)
 #include <QtQuick/private/qquickaccessibleattached_p.h>
@@ -160,11 +160,19 @@ bool QQuickControlPrivate::acceptTouch(const QTouchEvent::TouchPoint &point)
 }
 #endif
 
+static void setActiveFocus(QQuickControl *control, Qt::FocusReason reason)
+{
+    QQuickControlPrivate *d = QQuickControlPrivate::get(control);
+    if (d->subFocusItem && d->window && d->flags & QQuickItem::ItemIsFocusScope)
+        QQuickWindowPrivate::get(d->window)->clearFocusInScope(control, d->subFocusItem, reason);
+    control->forceActiveFocus(reason);
+}
+
 void QQuickControlPrivate::handlePress(const QPointF &)
 {
     Q_Q(QQuickControl);
     if ((focusPolicy & Qt::ClickFocus) == Qt::ClickFocus && !QGuiApplication::styleHints()->setFocusOnTouchRelease())
-        q->forceActiveFocus(Qt::MouseFocusReason);
+        setActiveFocus(q, Qt::MouseFocusReason);
 }
 
 void QQuickControlPrivate::handleMove(const QPointF &point)
@@ -181,7 +189,7 @@ void QQuickControlPrivate::handleRelease(const QPointF &)
 {
     Q_Q(QQuickControl);
     if ((focusPolicy & Qt::ClickFocus) == Qt::ClickFocus && QGuiApplication::styleHints()->setFocusOnTouchRelease())
-        q->forceActiveFocus(Qt::MouseFocusReason);
+        setActiveFocus(q, Qt::MouseFocusReason);
     touchId = -1;
 }
 
@@ -280,6 +288,8 @@ void QQuickControlPrivate::resizeContent()
 
 QQuickItem *QQuickControlPrivate::getContentItem()
 {
+    if (!contentItem)
+        executeContentItem();
     return contentItem;
 }
 
@@ -289,8 +299,11 @@ void QQuickControlPrivate::setContentItem_helper(QQuickItem *item, bool notify)
     if (contentItem == item)
         return;
 
+    if (!contentItem.isExecuting())
+        cancelContentItem();
+
     q->contentItemChange(item, contentItem);
-    destroyDelegate(contentItem, q);
+    delete contentItem;
     contentItem = item;
 
     if (item) {
@@ -300,7 +313,7 @@ void QQuickControlPrivate::setContentItem_helper(QQuickItem *item, bool notify)
             resizeContent();
     }
 
-    if (notify)
+    if (notify && !contentItem.isExecuting())
         emit q->contentItemChanged();
 }
 
@@ -569,8 +582,9 @@ void QQuickControl::itemChange(QQuickItem::ItemChange change, const QQuickItem::
             setHovered(false);
 #endif
         break;
+    case ItemSceneChange:
     case ItemParentHasChanged:
-        if (value.item) {
+        if ((change == ItemParentHasChanged && value.item) || (change == ItemSceneChange && value.window)) {
             d->resolveFont();
             if (!d->hasLocale)
                 d->updateLocale(QQuickControlPrivate::calcLocale(d->parentItem), false); // explicit=false
@@ -932,26 +946,44 @@ QLocale QQuickControlPrivate::calcLocale(const QQuickItem *item)
     return QLocale();
 }
 
-/*
-    Cancels incubation recursively to avoid "Object destroyed during incubation" (QTBUG-50992)
-*/
-static void cancelIncubation(QObject *object, QQmlContext *context)
+static inline QString contentItemName() { return QStringLiteral("contentItem"); }
+
+void QQuickControlPrivate::cancelContentItem()
 {
-    const auto children = object->children();
-    for (QObject *child : children)
-        cancelIncubation(child, context);
-    QQmlIncubatorPrivate::cancel(object, context);
+    Q_Q(QQuickControl);
+    quickCancelDeferred(q, contentItemName());
 }
 
-void QQuickControlPrivate::destroyDelegate(QObject *delegate, QObject *parent)
+void QQuickControlPrivate::executeContentItem(bool complete)
 {
-    if (!delegate)
+    Q_Q(QQuickControl);
+    if (contentItem.wasExecuted())
         return;
 
-    QQmlContext *context = parent ? qmlContext(parent) : nullptr;
-    if (context)
-        cancelIncubation(delegate, context);
-    delete delegate;
+    if (!contentItem || complete)
+        quickBeginDeferred(q, contentItemName(), contentItem);
+    if (complete)
+        quickCompleteDeferred(q, contentItemName(), contentItem);
+}
+
+static inline QString backgroundName() { return QStringLiteral("background"); }
+
+void QQuickControlPrivate::cancelBackground()
+{
+    Q_Q(QQuickControl);
+    quickCancelDeferred(q, backgroundName());
+}
+
+void QQuickControlPrivate::executeBackground(bool complete)
+{
+    Q_Q(QQuickControl);
+    if (background.wasExecuted())
+        return;
+
+    if (!background || complete)
+        quickBeginDeferred(q, backgroundName(), background);
+    if (complete)
+        quickCompleteDeferred(q, backgroundName(), background);
 }
 
 void QQuickControlPrivate::updateLocale(const QLocale &l, bool e)
@@ -1213,7 +1245,9 @@ void QQuickControl::setWheelEnabled(bool enabled)
 */
 QQuickItem *QQuickControl::background() const
 {
-    Q_D(const QQuickControl);
+    QQuickControlPrivate *d = const_cast<QQuickControlPrivate *>(d_func());
+    if (!d->background)
+        d->executeBackground();
     return d->background;
 }
 
@@ -1223,7 +1257,10 @@ void QQuickControl::setBackground(QQuickItem *background)
     if (d->background == background)
         return;
 
-    QQuickControlPrivate::destroyDelegate(d->background, this);
+    if (!d->background.isExecuting())
+        d->cancelBackground();
+
+    delete d->background;
     d->background = background;
     if (background) {
         background->setParentItem(this);
@@ -1232,7 +1269,8 @@ void QQuickControl::setBackground(QQuickItem *background)
         if (isComponentComplete())
             d->resizeBackground();
     }
-    emit backgroundChanged();
+    if (!d->background.isExecuting())
+        emit backgroundChanged();
 }
 
 /*!
@@ -1287,7 +1325,10 @@ void QQuickControl::classBegin()
 void QQuickControl::componentComplete()
 {
     Q_D(QQuickControl);
+    d->executeBackground(true);
+    d->executeContentItem(true);
     QQuickItem::componentComplete();
+    d->resizeBackground();
     d->resizeContent();
     if (!d->hasLocale)
         d->locale = QQuickControlPrivate::calcLocale(d->parentItem);
@@ -1374,13 +1415,8 @@ void QQuickControl::touchEvent(QTouchEvent *event)
     Q_D(QQuickControl);
     switch (event->type()) {
     case QEvent::TouchBegin:
-        for (const QTouchEvent::TouchPoint &point : event->touchPoints()) {
-            if (d->acceptTouch(point))
-                d->handlePress(point.pos());
-        }
-        break;
-
     case QEvent::TouchUpdate:
+    case QEvent::TouchEnd:
         for (const QTouchEvent::TouchPoint &point : event->touchPoints()) {
             if (!d->acceptTouch(point))
                 continue;
@@ -1401,13 +1437,6 @@ void QQuickControl::touchEvent(QTouchEvent *event)
         }
         break;
 
-    case QEvent::TouchEnd:
-        for (const QTouchEvent::TouchPoint &point : event->touchPoints()) {
-            if (d->acceptTouch(point))
-                d->handleRelease(point.pos());
-        }
-        break;
-
     case QEvent::TouchCancel:
         d->handleUngrab();
         break;
@@ -1430,7 +1459,7 @@ void QQuickControl::wheelEvent(QWheelEvent *event)
 {
     Q_D(QQuickControl);
     if ((d->focusPolicy & Qt::WheelFocus) == Qt::WheelFocus)
-        forceActiveFocus(Qt::MouseFocusReason);
+        setActiveFocus(this, Qt::MouseFocusReason);
 
     event->setAccepted(d->wheelEnabled);
 }
