@@ -40,6 +40,7 @@
 
 #include <QtGui/qstylehints.h>
 #include <QtGui/private/qguiapplication_p.h>
+#include <QtQml/qqmlinfo.h>
 #include <QtQuick/private/qquickwindow_p.h>
 #include <QtQuick/private/qquickanimation_p.h>
 #include <QtQuick/private/qquicktransition_p.h>
@@ -78,6 +79,11 @@ QT_BEGIN_NAMESPACE
             id: drawer
             width: 0.66 * window.width
             height: window.height
+
+            Label {
+                text: "Content goes here!"
+                anchors.centerIn: parent
+            }
         }
     }
     \endcode
@@ -152,6 +158,11 @@ QT_BEGIN_NAMESPACE
     If you would like the application's contents to stay where they are when
     the drawer is opened, don't apply a translation.
 
+    Drawer can be configured as a non-closable persistent side panel by
+    making the Drawer \l {Popup::modal}{non-modal} and \l {interactive}
+    {non-interactive}. See the \l {Qt Quick Controls 2 - Side Panel}{Side Panel}
+    example for more details.
+
     \note On some platforms, certain edges may be reserved for system
     gestures and therefore cannot be used with Drawer. For example, the
     top and bottom edges may be reserved for system notifications and
@@ -245,7 +256,7 @@ void QQuickDrawerPrivate::resizeOverlay()
     dimmer->setSize(geometry.size());
 }
 
-static bool isWithinDragMargin(QQuickDrawer *drawer, const QPointF &pos)
+static bool isWithinDragMargin(const QQuickDrawer *drawer, const QPointF &pos)
 {
     switch (drawer->edge()) {
     case Qt::LeftEdge:
@@ -346,10 +357,10 @@ bool QQuickDrawerPrivate::grabMouse(QQuickItem *item, QMouseEvent *event)
 bool QQuickDrawerPrivate::grabTouch(QQuickItem *item, QTouchEvent *event)
 {
     Q_Q(QQuickDrawer);
-    handleTouchEvent(item, event);
+    bool handled = handleTouchEvent(item, event);
 
     if (!window || !interactive || popupItem->keepTouchGrab() || !event->touchPointStates().testFlag(Qt::TouchPointMoved))
-        return false;
+        return handled;
 
     bool overThreshold = false;
     for (const QTouchEvent::TouchPoint &point : event->touchPoints()) {
@@ -392,6 +403,30 @@ bool QQuickDrawerPrivate::grabTouch(QQuickItem *item, QTouchEvent *event)
 
 static const qreal openCloseVelocityThreshold = 300;
 
+bool QQuickDrawerPrivate::blockInput(QQuickItem *item, const QPointF &point) const
+{
+    Q_Q(const QQuickDrawer);
+
+    // We want all events, if mouse/touch is already grabbed.
+    if (popupItem->keepMouseGrab() || popupItem->keepTouchGrab())
+        return true;
+
+    // Don't block input to drawer's children/content.
+    if (popupItem->isAncestorOf(item))
+        return false;
+
+    // Don't block outside a drawer's background dimming
+    if (dimmer && !dimmer->contains(dimmer->mapFromScene(point)))
+        return false;
+
+    // Accept all events within drag area.
+    if (isWithinDragMargin(q, point))
+        return true;
+
+    // Accept all other events if drawer is modal.
+    return modal;
+}
+
 bool QQuickDrawerPrivate::handlePress(QQuickItem *item, const QPointF &point, ulong timestamp)
 {
     offset = 0;
@@ -410,7 +445,7 @@ bool QQuickDrawerPrivate::handleMove(QQuickItem *item, const QPointF &point, ulo
         return false;
 
     // limit/reset the offset to the edge of the drawer when pushed from the outside
-    if (qFuzzyCompare(position, 1.0) && !contains(point))
+    if (qFuzzyCompare(position, qreal(1.0)) && !contains(point))
         offset = 0;
 
     bool isGrabbed = popupItem->keepMouseGrab() || popupItem->keepTouchGrab();
@@ -530,20 +565,32 @@ bool QQuickDrawerPrivate::prepareExitTransition()
     return QQuickPopupPrivate::prepareExitTransition();
 }
 
-void QQuickDrawerPrivate::setEdge(Qt::Edge e)
+bool QQuickDrawerPrivate::setEdge(Qt::Edge e)
 {
-    edge = e;
-    if (edge == Qt::LeftEdge || edge == Qt::RightEdge) {
+    Q_Q(QQuickDrawer);
+    switch (e) {
+    case Qt::LeftEdge:
+    case Qt::RightEdge:
         allowVerticalMove = true;
         allowVerticalResize = true;
         allowHorizontalMove = false;
         allowHorizontalResize = false;
-    } else {
+        break;
+    case Qt::TopEdge:
+    case Qt::BottomEdge:
         allowVerticalMove = false;
         allowVerticalResize = false;
         allowHorizontalMove = true;
         allowHorizontalResize = true;
+        break;
+    default:
+        qmlWarning(q) << "invalid edge value - valid values are: "
+            << "Qt.TopEdge, Qt.LeftEdge, Qt.RightEdge, Qt.BottomEdge";
+        return false;
     }
+
+    edge = e;
+    return true;
 }
 
 QQuickDrawer::QQuickDrawer(QObject *parent)
@@ -578,7 +625,9 @@ void QQuickDrawer::setEdge(Qt::Edge edge)
     if (d->edge == edge)
         return;
 
-    d->setEdge(edge);
+    if (!d->setEdge(edge))
+        return;
+
     if (isComponentComplete())
         d->reposition();
     emit edgeChanged();
