@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2016 The Qt Company Ltd.
+** Copyright (C) 2017 The Qt Company Ltd.
 ** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the Qt Quick Templates 2 module of the Qt Toolkit.
@@ -36,6 +36,7 @@
 
 #include "qquickgroupbox_p.h"
 #include "qquickframe_p_p.h"
+#include "qquickdeferredexecute_p_p.h"
 
 #include <QtGui/qpa/qplatformtheme.h>
 
@@ -48,7 +49,7 @@ QT_BEGIN_NAMESPACE
     \inqmlmodule QtQuick.Controls
     \since 5.7
     \ingroup qtquickcontrols2-containers
-    \brief A logical group of controls within a titled visual frame.
+    \brief Visual frame and title for a logical group of controls.
 
     GroupBox is used to layout a logical group of controls together, within
     a \l {title}{titled} visual frame. GroupBox does not provide a layout of its own, but
@@ -56,8 +57,8 @@ QT_BEGIN_NAMESPACE
     or a \l ColumnLayout.
 
     Items declared as children of a GroupBox are automatically parented to the
-    GroupBox's contentItem. Items created dynamically need to be explicitly
-    parented to the contentItem.
+    GroupBox's \l {Control::}{contentItem}. Items created dynamically need to be
+    explicitly parented to the contentItem.
 
     If only a single item is used within a GroupBox, it will resize to fit the
     implicit size of its contained item. This makes it particularly suitable
@@ -85,15 +86,40 @@ QT_BEGIN_NAMESPACE
 
 class QQuickGroupBoxPrivate : public QQuickFramePrivate
 {
+    Q_DECLARE_PUBLIC(QQuickGroupBox)
+
 public:
     QQuickGroupBoxPrivate() : label(nullptr) { }
 
+    void cancelLabel();
+    void executeLabel(bool complete = false);
+
     QString title;
-    QQuickItem *label;
+    QQuickDeferredPointer<QQuickItem> label;
 };
 
-QQuickGroupBox::QQuickGroupBox(QQuickItem *parent) :
-    QQuickFrame(*(new QQuickGroupBoxPrivate), parent)
+static inline QString labelName() { return QStringLiteral("label"); }
+
+void QQuickGroupBoxPrivate::cancelLabel()
+{
+    Q_Q(QQuickGroupBox);
+    quickCancelDeferred(q, labelName());
+}
+
+void QQuickGroupBoxPrivate::executeLabel(bool complete)
+{
+    Q_Q(QQuickGroupBox);
+    if (label.wasExecuted())
+        return;
+
+    if (!label || complete)
+        quickBeginDeferred(q, labelName(), label);
+    if (complete)
+        quickCompleteDeferred(q, labelName(), label);
+}
+
+QQuickGroupBox::QQuickGroupBox(QQuickItem *parent)
+    : QQuickFrame(*(new QQuickGroupBoxPrivate), parent)
 {
 }
 
@@ -118,6 +144,7 @@ void QQuickGroupBox::setTitle(const QString &title)
         return;
 
     d->title = title;
+    setAccessibleName(title);
     emit titleChanged();
 }
 
@@ -130,7 +157,9 @@ void QQuickGroupBox::setTitle(const QString &title)
 */
 QQuickItem *QQuickGroupBox::label() const
 {
-    Q_D(const QQuickGroupBox);
+    QQuickGroupBoxPrivate *d = const_cast<QQuickGroupBoxPrivate *>(d_func());
+    if (!d->label)
+        d->executeLabel();
     return d->label;
 }
 
@@ -140,11 +169,22 @@ void QQuickGroupBox::setLabel(QQuickItem *label)
     if (d->label == label)
         return;
 
-    d->deleteDelegate(d->label);
+    if (!d->label.isExecuting())
+        d->cancelLabel();
+
+    delete d->label;
     d->label = label;
     if (label && !label->parentItem())
         label->setParentItem(this);
-    emit labelChanged();
+    if (!d->label.isExecuting())
+        emit labelChanged();
+}
+
+void QQuickGroupBox::componentComplete()
+{
+    Q_D(QQuickGroupBox);
+    d->executeLabel(true);
+    QQuickFrame::componentComplete();
 }
 
 QFont QQuickGroupBox::defaultFont() const
@@ -152,4 +192,20 @@ QFont QQuickGroupBox::defaultFont() const
     return QQuickControlPrivate::themeFont(QPlatformTheme::GroupBoxTitleFont);
 }
 
+#if QT_CONFIG(accessibility)
+QAccessible::Role QQuickGroupBox::accessibleRole() const
+{
+    return QAccessible::Grouping;
+}
+
+void QQuickGroupBox::accessibilityActiveChanged(bool active)
+{
+    Q_D(QQuickGroupBox);
+    QQuickFrame::accessibilityActiveChanged(active);
+
+    if (active)
+        setAccessibleName(d->title);
+}
+#endif
+
 QT_END_NAMESPACE
