@@ -79,8 +79,8 @@ static QQuickItem *createDimmer(QQmlComponent *component, QQuickPopup *popup, QQ
     if (component) {
         QQmlContext *creationContext = component->creationContext();
         if (!creationContext)
-            creationContext = qmlContext(parent);
-        QQmlContext *context = new QQmlContext(creationContext, parent);
+            creationContext = qmlContext(popup);
+        QQmlContext *context = new QQmlContext(creationContext, popup);
         context->setContextObject(popup);
         item = qobject_cast<QQuickItem*>(component->beginCreate(context));
     }
@@ -169,10 +169,9 @@ QVector<QQuickDrawer *> QQuickOverlayPrivate::stackingOrderDrawers() const
     return sorted;
 }
 
-void QQuickOverlayPrivate::itemGeometryChanged(QQuickItem *item, QQuickGeometryChange, const QRectF &)
+void QQuickOverlayPrivate::itemGeometryChanged(QQuickItem *, QQuickGeometryChange, const QRectF &)
 {
-    Q_Q(QQuickOverlay);
-    q->setSize(QSizeF(item->width(), item->height()));
+    updateGeometry();
 }
 
 QQuickOverlayPrivate::QQuickOverlayPrivate()
@@ -210,6 +209,11 @@ bool QQuickOverlayPrivate::startDrag(QEvent *event, const QPointF &pos)
     return false;
 }
 
+static bool isTouchEvent(QEvent *event)
+{
+    return event->type() == QEvent::TouchBegin || event->type() == QEvent::TouchUpdate || event->type() == QEvent::TouchEnd;
+}
+
 bool QQuickOverlayPrivate::handlePress(QQuickItem *source, QEvent *event, QQuickPopup *target)
 {
     if (target) {
@@ -218,7 +222,7 @@ bool QQuickOverlayPrivate::handlePress(QQuickItem *source, QEvent *event, QQuick
             return true;
         }
         return false;
-    } else if (!mouseGrabberPopup) {
+    } else if (!mouseGrabberPopup || isTouchEvent(event)) {
         // allow non-modal popups to close themselves,
         // and non-dimming modal popups to block the event
         const auto popups = stackingOrderPopups();
@@ -337,6 +341,43 @@ void QQuickOverlayPrivate::setMouseGrabberPopup(QQuickPopup *popup)
     mouseGrabberPopup = popup;
 }
 
+void QQuickOverlayPrivate::updateGeometry()
+{
+    Q_Q(QQuickOverlay);
+    if (!window)
+        return;
+
+    QPointF pos;
+    QSizeF size = window->size();
+    qreal rotation = 0;
+
+    switch (window->contentOrientation()) {
+    case Qt::PrimaryOrientation:
+    case Qt::PortraitOrientation:
+        size = window->size();
+        break;
+    case Qt::LandscapeOrientation:
+        rotation = 90;
+        pos = QPointF((size.width() - size.height()) / 2, -(size.width() - size.height()) / 2);
+        size.transpose();
+        break;
+    case Qt::InvertedPortraitOrientation:
+        rotation = 180;
+        break;
+    case Qt::InvertedLandscapeOrientation:
+        rotation = 270;
+        pos = QPointF((size.width() - size.height()) / 2, -(size.width() - size.height()) / 2);
+        size.transpose();
+        break;
+    default:
+        break;
+    }
+
+    q->setSize(size);
+    q->setPosition(pos);
+    q->setRotation(rotation);
+}
+
 QQuickOverlay::QQuickOverlay(QQuickItem *parent)
     : QQuickItem(*(new QQuickOverlayPrivate), parent)
 {
@@ -347,10 +388,12 @@ QQuickOverlay::QQuickOverlay(QQuickItem *parent)
     setVisible(false);
 
     if (parent) {
-        setSize(QSizeF(parent->width(), parent->height()));
+        d->updateGeometry();
         QQuickItemPrivate::get(parent)->addItemChangeListener(d, QQuickItemPrivate::Geometry);
-        if (QQuickWindow *window = parent->window())
+        if (QQuickWindow *window = parent->window()) {
             window->installEventFilter(this);
+            QObjectPrivate::connect(window, &QWindow::contentOrientationChanged, d, &QQuickOverlayPrivate::updateGeometry);
+        }
     }
 }
 
