@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2016 The Qt Company Ltd.
+** Copyright (C) 2017 The Qt Company Ltd.
 ** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the Qt Quick Templates 2 module of the Qt Toolkit.
@@ -36,10 +36,7 @@
 
 #include "qquickpage_p.h"
 #include "qquickcontrol_p_p.h"
-#include "qquicktoolbar_p.h"
-#include "qquicktabbar_p.h"
-
-#include <QtQuick/private/qquickitemchangelistener_p.h>
+#include "qquickpagelayout_p_p.h"
 
 QT_BEGIN_NAMESPACE
 
@@ -50,7 +47,7 @@ QT_BEGIN_NAMESPACE
     \inqmlmodule QtQuick.Controls
     \since 5.7
     \ingroup qtquickcontrols2-containers
-    \brief A control that makes it convenient to add a header and footer to a page.
+    \brief Styled page control with support for a header and footer.
 
     Page is a container control which makes it convenient to add
     a \l header and \l footer item to a page.
@@ -61,7 +58,7 @@ QT_BEGIN_NAMESPACE
     toolbar header and an application-wide tabbar footer.
 
     \qml
-    import QtQuick.Controls 2.0
+    import QtQuick.Controls 2.1
 
     ApplicationWindow {
         visible: true
@@ -85,81 +82,45 @@ QT_BEGIN_NAMESPACE
     \sa ApplicationWindow, {Container Controls}
 */
 
-class QQuickPagePrivate : public QQuickControlPrivate, public QQuickItemChangeListener
+class QQuickPagePrivate : public QQuickControlPrivate
 {
     Q_DECLARE_PUBLIC(QQuickPage)
 
 public:
     QQuickPagePrivate();
 
-    void relayout();
-
-    void itemGeometryChanged(QQuickItem *item, const QRectF &newRect, const QRectF &oldRect) override;
-    void itemVisibilityChanged(QQuickItem *item) override;
-    void itemImplicitWidthChanged(QQuickItem *item) override;
-    void itemImplicitHeightChanged(QQuickItem *item) override;
+    QQuickItem *getContentItem() override;
 
+    qreal contentWidth;
+    qreal contentHeight;
     QString title;
-    QQuickItem *header;
-    QQuickItem *footer;
+    QScopedPointer<QQuickPageLayout> layout;
 };
 
-QQuickPagePrivate::QQuickPagePrivate() : header(nullptr), footer(nullptr)
+QQuickPagePrivate::QQuickPagePrivate()
+    : contentWidth(0),
+      contentHeight(0)
 {
 }
 
-void QQuickPagePrivate::relayout()
+QQuickItem *QQuickPagePrivate::getContentItem()
 {
     Q_Q(QQuickPage);
-    QQuickItem *content = q->contentItem();
-    const qreal hh = header && header->isVisible() ? header->height() : 0;
-    const qreal fh = footer && footer->isVisible() ? footer->height() : 0;
-
-    content->setY(hh + q->topPadding());
-    content->setX(q->leftPadding());
-    content->setWidth(q->availableWidth());
-    content->setHeight(q->availableHeight() - hh - fh);
-
-    if (header)
-        header->setWidth(q->width());
-
-    if (footer) {
-        footer->setY(q->height() - fh);
-        footer->setWidth(q->width());
-    }
+    if (QQuickItem *item = QQuickControlPrivate::getContentItem())
+        return item;
+    return new QQuickItem(q);
 }
 
-void QQuickPagePrivate::itemGeometryChanged(QQuickItem *item, const QRectF &newRect, const QRectF &oldRect)
-{
-    Q_UNUSED(item)
-    Q_UNUSED(newRect)
-    Q_UNUSED(oldRect)
-    relayout();
-}
-
-void QQuickPagePrivate::itemVisibilityChanged(QQuickItem *item)
-{
-    Q_UNUSED(item);
-    relayout();
-}
-
-void QQuickPagePrivate::itemImplicitWidthChanged(QQuickItem *item)
-{
-    Q_UNUSED(item);
-    relayout();
-}
-
-void QQuickPagePrivate::itemImplicitHeightChanged(QQuickItem *item)
-{
-    Q_UNUSED(item);
-    relayout();
-}
-
-QQuickPage::QQuickPage(QQuickItem *parent) :
-    QQuickControl(*(new QQuickPagePrivate), parent)
+QQuickPage::QQuickPage(QQuickItem *parent)
+    : QQuickControl(*(new QQuickPagePrivate), parent)
 {
+    Q_D(QQuickPage);
     setFlag(ItemIsFocusScope);
     setAcceptedMouseButtons(Qt::AllButtons);
+#if QT_CONFIG(cursor)
+    setCursor(Qt::ArrowCursor);
+#endif
+    d->layout.reset(new QQuickPageLayout(this));
 }
 
 /*!
@@ -211,6 +172,7 @@ void QQuickPage::setTitle(const QString &title)
         return;
 
     d->title = title;
+    setAccessibleName(title);
     emit titleChanged();
 }
 
@@ -220,43 +182,25 @@ void QQuickPage::setTitle(const QString &title)
     This property holds the page header item. The header item is positioned to
     the top, and resized to the width of the page. The default value is \c null.
 
-    \note Assigning a ToolBar or TabBar as a page header sets the respective
-    \l ToolBar::position or \l TabBar::position property automatically to \c Header.
+    \note Assigning a ToolBar, TabBar, or DialogButtonBox as a page header
+    automatically sets the respective \l ToolBar::position, \l TabBar::position,
+    or \l DialogButtonBox::position property to \c Header.
 
     \sa footer, ApplicationWindow::header
 */
 QQuickItem *QQuickPage::header() const
 {
     Q_D(const QQuickPage);
-    return d->header;
+    return d->layout->header();
 }
 
 void QQuickPage::setHeader(QQuickItem *header)
 {
     Q_D(QQuickPage);
-    if (d->header == header)
+    if (!d->layout->setHeader(header))
         return;
-
-    if (d->header) {
-        QQuickItemPrivate::get(d->header)->removeItemChangeListener(d, QQuickItemPrivate::Geometry | QQuickItemPrivate::Visibility |
-                                                                    QQuickItemPrivate::ImplicitWidth | QQuickItemPrivate::ImplicitHeight);
-        d->header->setParentItem(nullptr);
-    }
-    d->header = header;
-    if (header) {
-        header->setParentItem(this);
-        QQuickItemPrivate *p = QQuickItemPrivate::get(header);
-        p->addItemChangeListener(d, QQuickItemPrivate::Geometry | QQuickItemPrivate::Visibility |
-                                 QQuickItemPrivate::ImplicitWidth | QQuickItemPrivate::ImplicitHeight);
-        if (qFuzzyIsNull(header->z()))
-            header->setZ(1);
-        if (QQuickToolBar *toolBar = qobject_cast<QQuickToolBar *>(header))
-            toolBar->setPosition(QQuickToolBar::Header);
-        else if (QQuickTabBar *tabBar = qobject_cast<QQuickTabBar *>(header))
-            tabBar->setPosition(QQuickTabBar::Header);
-    }
     if (isComponentComplete())
-        d->relayout();
+        d->layout->update();
     emit headerChanged();
 }
 
@@ -266,43 +210,25 @@ void QQuickPage::setHeader(QQuickItem *header)
     This property holds the page footer item. The footer item is positioned to
     the bottom, and resized to the width of the page. The default value is \c null.
 
-    \note Assigning a ToolBar or TabBar as a page footer sets the respective
-    \l ToolBar::position or \l TabBar::position property automatically to \c Footer.
+    \note Assigning a ToolBar, TabBar, or DialogButtonBox as a page footer
+    automatically sets the respective \l ToolBar::position, \l TabBar::position,
+    or \l DialogButtonBox::position property to \c Footer.
 
     \sa header, ApplicationWindow::footer
 */
 QQuickItem *QQuickPage::footer() const
 {
     Q_D(const QQuickPage);
-    return d->footer;
+    return d->layout->footer();
 }
 
 void QQuickPage::setFooter(QQuickItem *footer)
 {
     Q_D(QQuickPage);
-    if (d->footer == footer)
+    if (!d->layout->setFooter(footer))
         return;
-
-    if (d->footer) {
-        QQuickItemPrivate::get(d->footer)->removeItemChangeListener(d, QQuickItemPrivate::Geometry | QQuickItemPrivate::Visibility |
-                                                                    QQuickItemPrivate::ImplicitWidth | QQuickItemPrivate::ImplicitHeight);
-        d->footer->setParentItem(nullptr);
-    }
-    d->footer = footer;
-    if (footer) {
-        footer->setParentItem(this);
-        QQuickItemPrivate *p = QQuickItemPrivate::get(footer);
-        p->addItemChangeListener(d, QQuickItemPrivate::Geometry | QQuickItemPrivate::Visibility |
-                                 QQuickItemPrivate::ImplicitWidth | QQuickItemPrivate::ImplicitHeight);
-        if (qFuzzyIsNull(footer->z()))
-            footer->setZ(1);
-        if (QQuickToolBar *toolBar = qobject_cast<QQuickToolBar *>(footer))
-            toolBar->setPosition(QQuickToolBar::Footer);
-        else if (QQuickTabBar *tabBar = qobject_cast<QQuickTabBar *>(footer))
-            tabBar->setPosition(QQuickTabBar::Footer);
-    }
     if (isComponentComplete())
-        d->relayout();
+        d->layout->update();
     emit footerChanged();
 }
 
@@ -322,8 +248,7 @@ void QQuickPage::setFooter(QQuickItem *footer)
 */
 QQmlListProperty<QObject> QQuickPage::contentData()
 {
-    Q_D(QQuickPage);
-    return QQmlListProperty<QObject>(d->contentItem, nullptr,
+    return QQmlListProperty<QObject>(contentItem(), nullptr,
                                      QQuickItemPrivate::data_append,
                                      QQuickItemPrivate::data_count,
                                      QQuickItemPrivate::data_at,
@@ -345,14 +270,70 @@ QQmlListProperty<QObject> QQuickPage::contentData()
 */
 QQmlListProperty<QQuickItem> QQuickPage::contentChildren()
 {
-    Q_D(QQuickPage);
-    return QQmlListProperty<QQuickItem>(d->contentItem, nullptr,
+    return QQmlListProperty<QQuickItem>(contentItem(), nullptr,
                                         QQuickItemPrivate::children_append,
                                         QQuickItemPrivate::children_count,
                                         QQuickItemPrivate::children_at,
                                         QQuickItemPrivate::children_clear);
 }
 
+/*!
+    \qmlproperty real QtQuick.Controls::Page::contentWidth
+    \since QtQuick.Controls 2.1 (Qt 5.8)
+
+    This property holds the content width. It is used for calculating the total
+    implicit width of the page.
+
+    \sa contentHeight
+*/
+qreal QQuickPage::contentWidth() const
+{
+    Q_D(const QQuickPage);
+    return d->contentWidth;
+}
+
+void QQuickPage::setContentWidth(qreal width)
+{
+    Q_D(QQuickPage);
+    if (qFuzzyCompare(d->contentWidth, width))
+        return;
+
+    d->contentWidth = width;
+    emit contentWidthChanged();
+}
+
+/*!
+    \qmlproperty real QtQuick.Controls::Page::contentHeight
+    \since QtQuick.Controls 2.1 (Qt 5.8)
+
+    This property holds the content height. It is used for calculating the total
+    implicit height of the page.
+
+    \sa contentWidth
+*/
+qreal QQuickPage::contentHeight() const
+{
+    Q_D(const QQuickPage);
+    return d->contentHeight;
+}
+
+void QQuickPage::setContentHeight(qreal height)
+{
+    Q_D(QQuickPage);
+    if (qFuzzyCompare(d->contentHeight, height))
+        return;
+
+    d->contentHeight = height;
+    emit contentHeightChanged();
+}
+
+void QQuickPage::componentComplete()
+{
+    Q_D(QQuickPage);
+    QQuickControl::componentComplete();
+    d->layout->update();
+}
+
 void QQuickPage::contentItemChange(QQuickItem *newItem, QQuickItem *oldItem)
 {
     QQuickControl::contentItemChange(newItem, oldItem);
@@ -367,21 +348,37 @@ void QQuickPage::geometryChanged(const QRectF &newGeometry, const QRectF &oldGeo
 {
     Q_D(QQuickPage);
     QQuickControl::geometryChanged(newGeometry, oldGeometry);
-    d->relayout();
+    d->layout->update();
 }
 
 void QQuickPage::paddingChange(const QMarginsF &newPadding, const QMarginsF &oldPadding)
 {
     Q_D(QQuickPage);
     QQuickControl::paddingChange(newPadding, oldPadding);
-    d->relayout();
+    d->layout->update();
 }
 
-#ifndef QT_NO_ACCESSIBILITY
+void QQuickPage::spacingChange(qreal newSpacing, qreal oldSpacing)
+{
+    Q_D(QQuickPage);
+    QQuickControl::spacingChange(newSpacing, oldSpacing);
+    d->layout->update();
+}
+
+#if QT_CONFIG(accessibility)
 QAccessible::Role QQuickPage::accessibleRole() const
 {
     return QAccessible::PageTab;
 }
+
+void QQuickPage::accessibilityActiveChanged(bool active)
+{
+    Q_D(QQuickPage);
+    QQuickControl::accessibilityActiveChanged(active);
+
+    if (active)
+        setAccessibleName(d->title);
+}
 #endif
 
 QT_END_NAMESPACE
