@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2016 The Qt Company Ltd.
+** Copyright (C) 2017 The Qt Company Ltd.
 ** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the Qt Quick Templates 2 module of the Qt Toolkit.
@@ -104,30 +104,38 @@ QT_BEGIN_NAMESPACE
     \sa {Customizing Pane}, {Container Controls}
 */
 
-QQuickPanePrivate::QQuickPanePrivate() : contentWidth(0), contentHeight(0)
+QQuickPanePrivate::QQuickPanePrivate()
+    : contentWidth(0),
+      contentHeight(0)
 {
 }
 
 QQuickItem *QQuickPanePrivate::getContentItem()
 {
     Q_Q(QQuickPane);
-    if (!contentItem)
-        contentItem = new QQuickItem(q);
-    return contentItem;
+    if (QQuickItem *item = QQuickControlPrivate::getContentItem())
+        return item;
+    return new QQuickItem(q);
 }
 
-QQuickPane::QQuickPane(QQuickItem *parent) :
-    QQuickControl(*(new QQuickPanePrivate), parent)
+QQuickPane::QQuickPane(QQuickItem *parent)
+    : QQuickControl(*(new QQuickPanePrivate), parent)
 {
     setFlag(QQuickItem::ItemIsFocusScope);
     setAcceptedMouseButtons(Qt::AllButtons);
+#if QT_CONFIG(cursor)
+    setCursor(Qt::ArrowCursor);
+#endif
 }
 
-QQuickPane::QQuickPane(QQuickPanePrivate &dd, QQuickItem *parent) :
-    QQuickControl(dd, parent)
+QQuickPane::QQuickPane(QQuickPanePrivate &dd, QQuickItem *parent)
+    : QQuickControl(dd, parent)
 {
     setFlag(QQuickItem::ItemIsFocusScope);
     setAcceptedMouseButtons(Qt::AllButtons);
+#if QT_CONFIG(cursor)
+    setCursor(Qt::ArrowCursor);
+#endif
 }
 
 /*!
@@ -198,8 +206,7 @@ void QQuickPane::setContentHeight(qreal height)
 */
 QQmlListProperty<QObject> QQuickPane::contentData()
 {
-    Q_D(QQuickPane);
-    return QQmlListProperty<QObject>(d->getContentItem(), nullptr,
+    return QQmlListProperty<QObject>(contentItem(), nullptr,
                                      QQuickItemPrivate::data_append,
                                      QQuickItemPrivate::data_count,
                                      QQuickItemPrivate::data_at,
@@ -221,8 +228,7 @@ QQmlListProperty<QObject> QQuickPane::contentData()
 */
 QQmlListProperty<QQuickItem> QQuickPane::contentChildren()
 {
-    Q_D(QQuickPane);
-    return QQmlListProperty<QQuickItem>(d->getContentItem(), nullptr,
+    return QQmlListProperty<QQuickItem>(contentItem(), nullptr,
                                         QQuickItemPrivate::children_append,
                                         QQuickItemPrivate::children_count,
                                         QQuickItemPrivate::children_at,
@@ -239,7 +245,7 @@ void QQuickPane::contentItemChange(QQuickItem *newItem, QQuickItem *oldItem)
     emit contentChildrenChanged();
 }
 
-#ifndef QT_NO_ACCESSIBILITY
+#if QT_CONFIG(accessibility)
 QAccessible::Role QQuickPane::accessibleRole() const
 {
     return QAccessible::Pane;
