@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2016 The Qt Company Ltd.
+** Copyright (C) 2017 The Qt Company Ltd.
 ** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the Qt Quick Templates 2 module of the Qt Toolkit.
@@ -36,6 +36,7 @@
 
 #include "qquickslider_p.h"
 #include "qquickcontrol_p_p.h"
+#include "qquickdeferredexecute_p_p.h"
 
 #include <QtQuick/private/qquickwindow_p.h>
 
@@ -48,64 +49,86 @@ QT_BEGIN_NAMESPACE
     \inqmlmodule QtQuick.Controls
     \since 5.7
     \ingroup qtquickcontrols2-input
-    \brief Selects a value by sliding a handle along a track.
+    \brief Used to select a value by sliding a handle along a track.
 
     \image qtquickcontrols2-slider.gif
 
     Slider is used to select a value by sliding a handle along a track.
 
-    \table
-    \row \li \image qtquickcontrols2-slider-normal.png
-         \li A slider in its normal state.
-    \row \li \image qtquickcontrols2-slider-focused.png
-         \li A slider that has active focus.
-    \row \li \image qtquickcontrols2-slider-disabled.png
-         \li A slider that is disabled.
-    \endtable
+    In the example below, custom \l from, \l value, and \l to values are set:
 
     \code
     Slider {
-        value: 0.5
+        from: 1
+        value: 25
+        to: 100
     }
     \endcode
 
+    The \l position property is expressed as a fraction of the control's size,
+    in the range \c {0.0 - 1.0}. The \l visualPosition property is
+    the same, except that it is reversed in a
+    \l {Right-to-left User Interfaces}{right-to-left} application. The
+    visualPosition is useful for positioning the handle when styling Slider.
+    In the example above, \l visualPosition will be \c 0.24 in a left-to-right
+    application, and \c 0.76 in a right-to-left application.
+
     \sa {Customizing Slider}, {Input Controls}
 */
 
+/*!
+    \since QtQuick.Controls 2.2 (Qt 5.9)
+    \qmlsignal QtQuick.Controls::Slider::moved()
+
+    This signal is emitted when the slider has been interactively moved
+    by the user by either touch, mouse, wheel, or keys.
+*/
+
 class QQuickSliderPrivate : public QQuickControlPrivate
 {
     Q_DECLARE_PUBLIC(QQuickSlider)
 
 public:
-    QQuickSliderPrivate() : from(0), to(1), value(0), position(0), stepSize(0), pressed(false),
-        orientation(Qt::Horizontal), snapMode(QQuickSlider::NoSnap),
-        handle(nullptr)
+    QQuickSliderPrivate()
+        : from(0),
+          to(1),
+          value(0),
+          position(0),
+          stepSize(0),
+          live(true),
+          pressed(false),
+          orientation(Qt::Horizontal),
+          snapMode(QQuickSlider::NoSnap),
+          handle(nullptr)
     {
     }
 
-    qreal valueAt(qreal position) const;
     qreal snapPosition(qreal position) const;
-    qreal positionAt(const QPoint &point) const;
+    qreal positionAt(const QPointF &point) const;
     void setPosition(qreal position);
     void updatePosition();
 
+    void handlePress(const QPointF &point) override;
+    void handleMove(const QPointF &point) override;
+    void handleRelease(const QPointF &point) override;
+    void handleUngrab() override;
+
+    void cancelHandle();
+    void executeHandle(bool complete = false);
+
     qreal from;
     qreal to;
     qreal value;
     qreal position;
     qreal stepSize;
+    bool live;
     bool pressed;
-    QPoint pressPoint;
+    QPointF pressPoint;
     Qt::Orientation orientation;
     QQuickSlider::SnapMode snapMode;
-    QQuickItem *handle;
+    QQuickDeferredPointer<QQuickItem> handle;
 };
 
-qreal QQuickSliderPrivate::valueAt(qreal position) const
-{
-    return from + (to - from) * position;
-}
-
 qreal QQuickSliderPrivate::snapPosition(qreal position) const
 {
     const qreal range = to - from;
@@ -119,26 +142,28 @@ qreal QQuickSliderPrivate::snapPosition(qreal position) const
     return qRound(position / effectiveStep) * effectiveStep;
 }
 
-qreal QQuickSliderPrivate::positionAt(const QPoint &point) const
+qreal QQuickSliderPrivate::positionAt(const QPointF &point) const
 {
     Q_Q(const QQuickSlider);
+    qreal pos = 0.0;
     if (orientation == Qt::Horizontal) {
         const qreal hw = handle ? handle->width() : 0;
         const qreal offset = hw / 2;
         const qreal extent = q->availableWidth() - hw;
         if (!qFuzzyIsNull(extent)) {
             if (q->isMirrored())
-                return (q->width() - point.x() - q->rightPadding() - offset) / extent;
-            return (point.x() - q->leftPadding() - offset) / extent;
+                pos = (q->width() - point.x() - q->rightPadding() - offset) / extent;
+            else
+                pos = (point.x() - q->leftPadding() - offset) / extent;
         }
     } else {
         const qreal hh = handle ? handle->height() : 0;
         const qreal offset = hh / 2;
         const qreal extent = q->availableHeight() - hh;
         if (!qFuzzyIsNull(extent))
-            return (q->height() - point.y() - q->bottomPadding() - offset) / extent;
+            pos = (q->height() - point.y() - q->bottomPadding() - offset) / extent;
     }
-    return 0;
+    return qBound<qreal>(0.0, pos, 1.0);
 }
 
 void QQuickSliderPrivate::setPosition(qreal pos)
@@ -161,12 +186,88 @@ void QQuickSliderPrivate::updatePosition()
     setPosition(pos);
 }
 
-QQuickSlider::QQuickSlider(QQuickItem *parent) :
-    QQuickControl(*(new QQuickSliderPrivate), parent)
+void QQuickSliderPrivate::handlePress(const QPointF &point)
+{
+    Q_Q(QQuickSlider);
+    QQuickControlPrivate::handlePress(point);
+    pressPoint = point;
+    q->setPressed(true);
+}
+
+void QQuickSliderPrivate::handleMove(const QPointF &point)
+{
+    Q_Q(QQuickSlider);
+    QQuickControlPrivate::handleMove(point);
+    const qreal oldPos = position;
+    qreal pos = positionAt(point);
+    if (snapMode == QQuickSlider::SnapAlways)
+        pos = snapPosition(pos);
+    if (live)
+        q->setValue(q->valueAt(pos));
+    else
+        setPosition(pos);
+    if (!qFuzzyCompare(pos, oldPos))
+        emit q->moved();
+}
+
+void QQuickSliderPrivate::handleRelease(const QPointF &point)
+{
+    Q_Q(QQuickSlider);
+    QQuickControlPrivate::handleRelease(point);
+    pressPoint = QPointF();
+    const qreal oldPos = position;
+    qreal pos = positionAt(point);
+    if (snapMode != QQuickSlider::NoSnap)
+        pos = snapPosition(pos);
+    qreal val = q->valueAt(pos);
+    if (!qFuzzyCompare(val, value))
+        q->setValue(val);
+    else if (snapMode != QQuickSlider::NoSnap)
+        setPosition(pos);
+    if (!qFuzzyCompare(pos, oldPos))
+        emit q->moved();
+    q->setKeepMouseGrab(false);
+    q->setKeepTouchGrab(false);
+    q->setPressed(false);
+}
+
+void QQuickSliderPrivate::handleUngrab()
+{
+    Q_Q(QQuickSlider);
+    QQuickControlPrivate::handleUngrab();
+    pressPoint = QPointF();
+    q->setPressed(false);
+}
+
+static inline QString handleName() { return QStringLiteral("handle"); }
+
+void QQuickSliderPrivate::cancelHandle()
+{
+    Q_Q(QQuickSlider);
+    quickCancelDeferred(q, handleName());
+}
+
+void QQuickSliderPrivate::executeHandle(bool complete)
+{
+    Q_Q(QQuickSlider);
+    if (handle.wasExecuted())
+        return;
+
+    if (!handle || complete)
+        quickBeginDeferred(q, handleName(), handle);
+    if (complete)
+        quickCompleteDeferred(q, handleName(), handle);
+}
+
+QQuickSlider::QQuickSlider(QQuickItem *parent)
+    : QQuickControl(*(new QQuickSliderPrivate), parent)
 {
     setActiveFocusOnTab(true);
     setFocusPolicy(Qt::StrongFocus);
     setAcceptedMouseButtons(Qt::LeftButton);
+#if QT_CONFIG(cursor)
+    setCursor(Qt::ArrowCursor);
+#endif
 }
 
 /*!
@@ -228,10 +329,6 @@ void QQuickSlider::setTo(qreal to)
 
     This property holds the value in the range \c from - \c to. The default value is \c 0.0.
 
-    Unlike the \l position property, the \c value is not updated while the
-    handle is dragged. The value is updated after the value has been chosen
-    and the slider has been released.
-
     \sa position
 */
 qreal QQuickSlider::value() const
@@ -260,12 +357,11 @@ void QQuickSlider::setValue(qreal value)
 
     This property holds the logical position of the handle.
 
-    The position is defined as a percentage of the control's size, scaled
-    to \c {0.0 - 1.0}. Unlike the \l value property, the \c position is
-    continuously updated while the handle is dragged. For visualizing a
-    slider, the right-to-left aware \l visualPosition should be used instead.
+    The position is expressed as a fraction of the control's size, in the range
+    \c {0.0 - 1.0}. For visualizing a slider, the right-to-left aware
+    \l visualPosition should be used instead.
 
-    \sa value, visualPosition
+    \sa value, visualPosition, valueAt()
 */
 qreal QQuickSlider::position() const
 {
@@ -279,7 +375,7 @@ qreal QQuickSlider::position() const
 
     This property holds the visual position of the handle.
 
-    The position is defined as a percentage of the control's size, scaled to
+    The position is expressed as a fraction of the control's size, in the range
     \c {0.0 - 1.0}. When the control is \l {Control::mirrored}{mirrored}, the
     value is equal to \c {1.0 - position}. This makes the value suitable for
     visualizing the slider, taking right-to-left support into account.
@@ -327,6 +423,18 @@ void QQuickSlider::setStepSize(qreal step)
     \value Slider.SnapAlways The slider snaps while the handle is dragged.
     \value Slider.SnapOnRelease The slider does not snap while being dragged, but only after the handle is released.
 
+    In the following table, the various modes are illustrated with animations.
+    The movement of the mouse cursor and the \l stepSize (\c 0.2) are identical
+    in each animation.
+
+    \table
+    \header
+        \row \li \b Value \li \b Example
+        \row \li \c Slider.NoSnap \li \image qtquickcontrols2-slider-nosnap.gif
+        \row \li \c Slider.SnapAlways \li \image qtquickcontrols2-slider-snapalways.gif
+        \row \li \c Slider.SnapOnRelease \li \image qtquickcontrols2-slider-snaponrelease.gif
+    \endtable
+
     \sa stepSize
 */
 QQuickSlider::SnapMode QQuickSlider::snapMode() const
@@ -401,7 +509,9 @@ void QQuickSlider::setOrientation(Qt::Orientation orientation)
 */
 QQuickItem *QQuickSlider::handle() const
 {
-    Q_D(const QQuickSlider);
+    QQuickSliderPrivate *d = const_cast<QQuickSliderPrivate *>(d_func());
+    if (!d->handle)
+        d->executeHandle();
     return d->handle;
 }
 
@@ -411,11 +521,59 @@ void QQuickSlider::setHandle(QQuickItem *handle)
     if (d->handle == handle)
         return;
 
-    d->deleteDelegate(d->handle);
+    if (!d->handle.isExecuting())
+        d->cancelHandle();
+
+    delete d->handle;
     d->handle = handle;
     if (handle && !handle->parentItem())
         handle->setParentItem(this);
-    emit handleChanged();
+    if (!d->handle.isExecuting())
+        emit handleChanged();
+}
+
+/*!
+    \since QtQuick.Controls 2.1 (Qt 5.8)
+    \qmlmethod real QtQuick.Controls::Slider::valueAt(real position)
+
+    Returns the value for the given \a position.
+
+    \sa value, position
+*/
+qreal QQuickSlider::valueAt(qreal position) const
+{
+    Q_D(const QQuickSlider);
+    const qreal value = (d->to - d->from) * position;
+    if (qFuzzyIsNull(d->stepSize))
+        return d->from + value;
+    return d->from + qRound(value / d->stepSize) * d->stepSize;
+}
+
+/*!
+    \since QtQuick.Controls 2.2 (Qt 5.9)
+    \qmlproperty bool QtQuick.Controls::Slider::live
+
+    This property holds whether the slider provides live updates for the \l value
+    property while the handle is dragged.
+
+    The default value is \c true.
+
+    \sa value, valueAt()
+*/
+bool QQuickSlider::live() const
+{
+    Q_D(const QQuickSlider);
+    return d->live;
+}
+
+void QQuickSlider::setLive(bool live)
+{
+    Q_D(QQuickSlider);
+    if (d->live == live)
+        return;
+
+    d->live = live;
+    emit liveChanged();
 }
 
 /*!
@@ -450,6 +608,8 @@ void QQuickSlider::keyPressEvent(QKeyEvent *event)
 {
     Q_D(QQuickSlider);
     QQuickControl::keyPressEvent(event);
+
+    const qreal oldValue = d->value;
     if (d->orientation == Qt::Horizontal) {
         if (event->key() == Qt::Key_Left) {
             setPressed(true);
@@ -477,6 +637,8 @@ void QQuickSlider::keyPressEvent(QKeyEvent *event)
             event->accept();
         }
     }
+    if (!qFuzzyCompare(d->value, oldValue))
+        emit moved();
 }
 
 void QQuickSlider::keyReleaseEvent(QKeyEvent *event)
@@ -489,53 +651,62 @@ void QQuickSlider::mousePressEvent(QMouseEvent *event)
 {
     Q_D(QQuickSlider);
     QQuickControl::mousePressEvent(event);
-    d->pressPoint = event->pos();
-    setPressed(true);
+    d->handleMove(event->localPos());
 }
 
 void QQuickSlider::mouseMoveEvent(QMouseEvent *event)
 {
     Q_D(QQuickSlider);
-    QQuickControl::mouseMoveEvent(event);
     if (!keepMouseGrab()) {
         if (d->orientation == Qt::Horizontal)
-            setKeepMouseGrab(QQuickWindowPrivate::dragOverThreshold(event->pos().x() - d->pressPoint.x(), Qt::XAxis, event));
+            setKeepMouseGrab(QQuickWindowPrivate::dragOverThreshold(event->localPos().x() - d->pressPoint.x(), Qt::XAxis, event));
         else
-            setKeepMouseGrab(QQuickWindowPrivate::dragOverThreshold(event->pos().y() - d->pressPoint.y(), Qt::YAxis, event));
-    }
-    if (keepMouseGrab()) {
-        qreal pos = d->positionAt(event->pos());
-        if (d->snapMode == SnapAlways)
-            pos = d->snapPosition(pos);
-        d->setPosition(pos);
+            setKeepMouseGrab(QQuickWindowPrivate::dragOverThreshold(event->localPos().y() - d->pressPoint.y(), Qt::YAxis, event));
     }
+    QQuickControl::mouseMoveEvent(event);
 }
 
-void QQuickSlider::mouseReleaseEvent(QMouseEvent *event)
+#if QT_CONFIG(quicktemplates2_multitouch)
+void QQuickSlider::touchEvent(QTouchEvent *event)
 {
     Q_D(QQuickSlider);
-    QQuickControl::mouseReleaseEvent(event);
-    d->pressPoint = QPoint();
-    qreal pos = d->positionAt(event->pos());
-    if (d->snapMode != NoSnap)
-        pos = d->snapPosition(pos);
-    qreal val = d->valueAt(pos);
-    if (!qFuzzyCompare(val, d->value))
-        setValue(val);
-    else if (d->snapMode != NoSnap)
-        d->setPosition(pos);
-    setKeepMouseGrab(false);
-    setPressed(false);
-}
+    switch (event->type()) {
+    case QEvent::TouchUpdate:
+        for (const QTouchEvent::TouchPoint &point : event->touchPoints()) {
+            if (!d->acceptTouch(point))
+                continue;
 
-void QQuickSlider::mouseUngrabEvent()
-{
-    Q_D(QQuickSlider);
-    QQuickControl::mouseUngrabEvent();
-    d->pressPoint = QPoint();
-    setPressed(false);
-}
+            switch (point.state()) {
+            case Qt::TouchPointPressed:
+                d->handlePress(point.pos());
+                break;
+            case Qt::TouchPointMoved:
+                if (!keepTouchGrab()) {
+                    if (d->orientation == Qt::Horizontal)
+                        setKeepTouchGrab(QQuickWindowPrivate::dragOverThreshold(point.pos().x() - d->pressPoint.x(), Qt::XAxis, &point));
+                    else
+                        setKeepTouchGrab(QQuickWindowPrivate::dragOverThreshold(point.pos().y() - d->pressPoint.y(), Qt::YAxis, &point));
+                }
+                if (keepTouchGrab())
+                    d->handleMove(point.pos());
+                break;
+            case Qt::TouchPointReleased:
+                d->handleRelease(point.pos());
+                break;
+            default:
+                break;
+            }
+        }
+        break;
 
+    default:
+        QQuickControl::touchEvent(event);
+        break;
+    }
+}
+#endif
+
+#if QT_CONFIG(wheelevent)
 void QQuickSlider::wheelEvent(QWheelEvent *event)
 {
     Q_D(QQuickSlider);
@@ -543,12 +714,16 @@ void QQuickSlider::wheelEvent(QWheelEvent *event)
     if (d->wheelEnabled) {
         const qreal oldValue = d->value;
         const QPointF angle = event->angleDelta();
-        const qreal delta = (qFuzzyIsNull(angle.y()) ? angle.x() : angle.y()) / QWheelEvent::DefaultDeltasPerStep;
+        const qreal delta = (qFuzzyIsNull(angle.y()) ? angle.x() : (event->inverted() ? -angle.y() : angle.y())) / QWheelEvent::DefaultDeltasPerStep;
         const qreal step = qFuzzyIsNull(d->stepSize) ? 0.1 : d->stepSize;
         setValue(oldValue + step * delta);
-        event->setAccepted(!qFuzzyCompare(d->value, oldValue));
+        const bool wasMoved = !qFuzzyCompare(d->value, oldValue);
+        if (wasMoved)
+            emit moved();
+        event->setAccepted(wasMoved);
     }
 }
+#endif
 
 void QQuickSlider::mirrorChange()
 {
@@ -559,12 +734,13 @@ void QQuickSlider::mirrorChange()
 void QQuickSlider::componentComplete()
 {
     Q_D(QQuickSlider);
+    d->executeHandle(true);
     QQuickControl::componentComplete();
     setValue(d->value);
     d->updatePosition();
 }
 
-#ifndef QT_NO_ACCESSIBILITY
+#if QT_CONFIG(accessibility)
 void QQuickSlider::accessibilityActiveChanged(bool active)
 {
     QQuickControl::accessibilityActiveChanged(active);
