@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2016 The Qt Company Ltd.
+** Copyright (C) 2017 The Qt Company Ltd.
 ** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the Qt Quick Templates 2 module of the Qt Toolkit.
@@ -106,7 +106,11 @@ class QQuickSwipeViewPrivate : public QQuickContainerPrivate
     Q_DECLARE_PUBLIC(QQuickSwipeView)
 
 public:
-    QQuickSwipeViewPrivate() : interactive(true) { }
+    QQuickSwipeViewPrivate()
+        : interactive(true),
+          orientation(Qt::Horizontal)
+    {
+    }
 
     void resizeItem(QQuickItem *item);
     void resizeItems();
@@ -114,6 +118,7 @@ public:
     static QQuickSwipeViewPrivate *get(QQuickSwipeView *view);
 
     bool interactive;
+    Qt::Orientation orientation;
 };
 
 void QQuickSwipeViewPrivate::resizeItems()
@@ -126,10 +131,14 @@ void QQuickSwipeViewPrivate::resizeItems()
             QQuickAnchors *anchors = QQuickItemPrivate::get(item)->_anchors;
             // TODO: expose QQuickAnchorLine so we can test for other conflicting anchors
             if (anchors && (anchors->fill() || anchors->centerIn()) && !item->property("_q_QQuickSwipeView_warned").toBool()) {
-                qmlInfo(item) << "SwipeView has detected conflicting anchors. Unable to layout the item.";
+                qmlWarning(item) << "SwipeView has detected conflicting anchors. Unable to layout the item.";
                 item->setProperty("_q_QQuickSwipeView_warned", true);
             }
 
+            if (orientation == Qt::Horizontal)
+                item->setY(0);
+            else
+                item->setX(0);
             item->setSize(QSizeF(contentItem->width(), contentItem->height()));
         }
     }
@@ -140,15 +149,15 @@ QQuickSwipeViewPrivate *QQuickSwipeViewPrivate::get(QQuickSwipeView *view)
     return view->d_func();
 }
 
-QQuickSwipeView::QQuickSwipeView(QQuickItem *parent) :
-    QQuickContainer(*(new QQuickSwipeViewPrivate), parent)
+QQuickSwipeView::QQuickSwipeView(QQuickItem *parent)
+    : QQuickContainer(*(new QQuickSwipeViewPrivate), parent)
 {
     setFlag(ItemIsFocusScope);
     setActiveFocusOnTab(true);
 }
 
 /*!
-    \since QtQuick.Controls 2.1
+    \since QtQuick.Controls 2.1 (Qt 5.8)
     \qmlproperty bool QtQuick.Controls::SwipeView::interactive
 
     This property describes whether the user can interact with the SwipeView.
@@ -172,6 +181,34 @@ void QQuickSwipeView::setInteractive(bool interactive)
     emit interactiveChanged();
 }
 
+/*!
+    \since QtQuick.Controls 2.2 (Qt 5.9)
+    \qmlproperty enumeration QtQuick.Controls::SwipeView::orientation
+
+    This property holds the orientation.
+
+    Possible values:
+    \value Qt.Horizontal Horizontal (default)
+    \value Qt.Vertical Vertical
+*/
+Qt::Orientation QQuickSwipeView::orientation() const
+{
+    Q_D(const QQuickSwipeView);
+    return d->orientation;
+}
+
+void QQuickSwipeView::setOrientation(Qt::Orientation orientation)
+{
+    Q_D(QQuickSwipeView);
+    if (d->orientation == orientation)
+        return;
+
+    d->orientation = orientation;
+    if (isComponentComplete())
+        d->resizeItems();
+    emit orientationChanged();
+}
+
 QQuickSwipeViewAttached *QQuickSwipeView::qmlAttachedProperties(QObject *object)
 {
     return new QQuickSwipeViewAttached(object);
@@ -192,7 +229,7 @@ void QQuickSwipeView::itemAdded(int, QQuickItem *item)
         item->setSize(QSizeF(d->contentItem->width(), d->contentItem->height()));
 }
 
-#ifndef QT_NO_ACCESSIBILITY
+#if QT_CONFIG(accessibility)
 QAccessible::Role QQuickSwipeView::accessibleRole() const
 {
     return QAccessible::PageTabList;
@@ -219,7 +256,7 @@ QAccessible::Role QQuickSwipeView::accessibleRole() const
 
 /*!
     \qmlattachedproperty bool QtQuick.Controls::SwipeView::isNextItem
-    \since QtQuick.Controls 2.1
+    \since QtQuick.Controls 2.1 (Qt 5.8)
     \readonly
 
     This attached property is \c true if this child is the next item.
@@ -229,7 +266,7 @@ QAccessible::Role QQuickSwipeView::accessibleRole() const
 
 /*!
     \qmlattachedproperty bool QtQuick.Controls::SwipeView::isPreviousItem
-    \since QtQuick.Controls 2.1
+    \since QtQuick.Controls 2.1 (Qt 5.8)
     \readonly
 
     This attached property is \c true if this child is the previous item.
@@ -250,11 +287,11 @@ class QQuickSwipeViewAttachedPrivate : public QObjectPrivate, public QQuickItemC
 {
     Q_DECLARE_PUBLIC(QQuickSwipeViewAttached)
 public:
-    QQuickSwipeViewAttachedPrivate() :
-        item(nullptr),
-        swipeView(nullptr),
-        index(-1),
-        currentIndex(-1)
+    QQuickSwipeViewAttachedPrivate()
+        : item(nullptr),
+          swipeView(nullptr),
+          index(-1),
+          currentIndex(-1)
     {
     }
 
@@ -395,8 +432,8 @@ void QQuickSwipeViewAttachedPrivate::itemDestroyed(QQuickItem *item)
     QQuickItemPrivate::get(item)->removeItemChangeListener(this, QQuickItemPrivate::Parent | QQuickItemPrivate::Destroyed);
 }
 
-QQuickSwipeViewAttached::QQuickSwipeViewAttached(QObject *parent) :
-    QObject(*(new QQuickSwipeViewAttachedPrivate), parent)
+QQuickSwipeViewAttached::QQuickSwipeViewAttached(QObject *parent)
+    : QObject(*(new QQuickSwipeViewAttachedPrivate), parent)
 {
     Q_D(QQuickSwipeViewAttached);
     d->item = qobject_cast<QQuickItem *>(parent);
@@ -407,7 +444,7 @@ QQuickSwipeViewAttached::QQuickSwipeViewAttached(QObject *parent) :
         QQuickItemPrivate *p = QQuickItemPrivate::get(d->item);
         p->addItemChangeListener(d, QQuickItemPrivate::Parent | QQuickItemPrivate::Destroyed);
     } else if (parent) {
-        qmlInfo(parent) << "SwipeView: attached properties must be accessed from within a child item";
+        qmlWarning(parent) << "SwipeView: attached properties must be accessed from within a child item";
     }
 }
 
@@ -419,12 +456,6 @@ QQuickSwipeViewAttached::~QQuickSwipeViewAttached()
         QQuickItemPrivate::get(item)->removeItemChangeListener(d, QQuickItemPrivate::Parent | QQuickItemPrivate::Destroyed);
 }
 
-QQuickSwipeView *QQuickSwipeViewAttached::view() const
-{
-    Q_D(const QQuickSwipeViewAttached);
-    return d->swipeView;
-}
-
 int QQuickSwipeViewAttached::index() const
 {
     Q_D(const QQuickSwipeViewAttached);
@@ -437,6 +468,12 @@ bool QQuickSwipeViewAttached::isCurrentItem() const
     return d->index != -1 && d->currentIndex != -1 && d->index == d->currentIndex;
 }
 
+QQuickSwipeView *QQuickSwipeViewAttached::view() const
+{
+    Q_D(const QQuickSwipeViewAttached);
+    return d->swipeView;
+}
+
 bool QQuickSwipeViewAttached::isNextItem() const
 {
     Q_D(const QQuickSwipeViewAttached);
