@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2016 The Qt Company Ltd.
+** Copyright (C) 2017 The Qt Company Ltd.
 ** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the Qt Quick Templates 2 module of the Qt Toolkit.
@@ -86,6 +86,10 @@ public:
 
     qreal positionAt(const QPointF &point) const;
 
+    bool canDrag(const QPointF &movePoint) const;
+    void handleMove(const QPointF &point) override;
+    void handleRelease(const QPointF &point) override;
+
     qreal position;
 };
 
@@ -100,8 +104,34 @@ qreal QQuickSwitchPrivate::positionAt(const QPointF &point) const
     return pos;
 }
 
-QQuickSwitch::QQuickSwitch(QQuickItem *parent) :
-    QQuickAbstractButton(*(new QQuickSwitchPrivate), parent)
+bool QQuickSwitchPrivate::canDrag(const QPointF &movePoint) const
+{
+    // don't start dragging the handle unless the initial press was at the indicator,
+    // or the drag has reached the indicator area. this prevents unnatural jumps when
+    // dragging far outside the indicator.
+    const qreal pressPos = positionAt(pressPoint);
+    const qreal movePos = positionAt(movePoint);
+    return (pressPos >= 0.0 && pressPos <= 1.0) || (movePos >= 0.0 && movePos <= 1.0);
+}
+
+void QQuickSwitchPrivate::handleMove(const QPointF &point)
+{
+    Q_Q(QQuickSwitch);
+    QQuickAbstractButtonPrivate::handleMove(point);
+    if (q->keepMouseGrab() || q->keepTouchGrab())
+        q->setPosition(positionAt(point));
+}
+
+void QQuickSwitchPrivate::handleRelease(const QPointF &point)
+{
+    Q_Q(QQuickSwitch);
+    QQuickAbstractButtonPrivate::handleRelease(point);
+    q->setKeepMouseGrab(false);
+    q->setKeepTouchGrab(false);
+}
+
+QQuickSwitch::QQuickSwitch(QQuickItem *parent)
+    : QQuickAbstractButton(*(new QQuickSwitchPrivate), parent)
 {
     Q_D(QQuickSwitch);
     d->keepPressed = true;
@@ -146,36 +176,32 @@ qreal QQuickSwitch::visualPosition() const
     return d->position;
 }
 
-void QQuickSwitch::mousePressEvent(QMouseEvent *event)
-{
-    QQuickAbstractButton::mousePressEvent(event);
-}
-
 void QQuickSwitch::mouseMoveEvent(QMouseEvent *event)
 {
     Q_D(QQuickSwitch);
-    QQuickAbstractButton::mouseMoveEvent(event);
-
-    const QPointF movePoint = event->localPos();
     if (!keepMouseGrab()) {
-        // don't start dragging the handle unless the initial press was at the indicator,
-        // or the drag has reached the indicator area. this prevents unnatural jumps when
-        // dragging far outside the indicator.
-        const qreal pressPos = d->positionAt(d->pressPoint);
-        const qreal movePos = d->positionAt(movePoint);
-        if ((pressPos >= 0.0 && pressPos <= 1.0) || (movePos >= 0.0 && movePos <= 1.0))
+        const QPointF movePoint = event->localPos();
+        if (d->canDrag(movePoint))
             setKeepMouseGrab(QQuickWindowPrivate::dragOverThreshold(movePoint.x() - d->pressPoint.x(), Qt::XAxis, event));
     }
-
-    if (keepMouseGrab())
-        setPosition(d->positionAt(movePoint));
+    QQuickAbstractButton::mouseMoveEvent(event);
 }
 
-void QQuickSwitch::mouseReleaseEvent(QMouseEvent *event)
+#if QT_CONFIG(quicktemplates2_multitouch)
+void QQuickSwitch::touchEvent(QTouchEvent *event)
 {
-    QQuickAbstractButton::mouseReleaseEvent(event);
-    setKeepMouseGrab(false);
+    Q_D(QQuickSwitch);
+    if (!keepTouchGrab() && event->type() == QEvent::TouchUpdate) {
+        for (const QTouchEvent::TouchPoint &point : event->touchPoints()) {
+            if (point.id() != d->touchId || point.state() != Qt::TouchPointMoved)
+                continue;
+            if (d->canDrag(point.pos()))
+                setKeepTouchGrab(QQuickWindowPrivate::dragOverThreshold(point.pos().x() - d->pressPoint.x(), Qt::XAxis, &point));
+        }
+    }
+    QQuickAbstractButton::touchEvent(event);
 }
+#endif
 
 void QQuickSwitch::mirrorChange()
 {
@@ -186,20 +212,23 @@ void QQuickSwitch::mirrorChange()
 void QQuickSwitch::nextCheckState()
 {
     Q_D(QQuickSwitch);
-    if (keepMouseGrab()) {
-        setChecked(d->position > 0.5);
+    if (keepMouseGrab() || keepTouchGrab()) {
+        d->toggle(d->position > 0.5);
         // the checked state might not change => force a position update to
         // avoid that the handle is left somewhere in the middle (QTBUG-57944)
-        checkStateSet();
+        setPosition(d->checked ? 1.0 : 0.0);
     } else {
         QQuickAbstractButton::nextCheckState();
     }
 }
 
-void QQuickSwitch::checkStateSet()
+void QQuickSwitch::buttonChange(ButtonChange change)
 {
     Q_D(QQuickSwitch);
-    setPosition(d->checked ? 1.0 : 0.0);
+    if (change == ButtonCheckedChange)
+        setPosition(d->checked ? 1.0 : 0.0);
+    else
+        QQuickAbstractButton::buttonChange(change);
 }
 
 QT_END_NAMESPACE
